package edu.mbhs.robot.scoutmaster9000.client.desktop;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Scanner;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.xml.bind.DatatypeConverter;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import edu.mbhs.robot.scoutmaster9000.client.desktop.imageUpload.ImageData;
import edu.mbhs.robot.scoutmaster9000.client.desktop.imageUpload.ImageManager;
import edu.mbhs.robot.scoutmaster9000.client.desktop.imageUpload.ImageManagerImpl;
import edu.mbhs.robot.scoutmaster9000.client.desktop.imageUpload.ImageSelector;

/**
 * The main frame for the desktop client
 * @author David Kaufman
 *
 */
public class ScoutMasterDesktopClientFrame extends JFrame {
	private static final long serialVersionUID = 1L;
	
	private ImageSelector imageSelector;
	private ImageManager imageManager;
	
	/**
	 * Sets up the frame for image-uploading (as of now the only task the desktop client is needed for)
	 */
	public ScoutMasterDesktopClientFrame() {
		super("Scoutmaster 9000 Desktop Client");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setMinimumSize(new Dimension(400,300));
	
		JPanel imagePanel = new JPanel();
		imagePanel.setLayout(new BoxLayout(imagePanel, BoxLayout.Y_AXIS));
		
		JPanel imagePanelDropbox = new JPanel();
		imagePanelDropbox.setLayout(new BoxLayout(imagePanelDropbox, BoxLayout.Y_AXIS));
		imagePanel.add(imagePanelDropbox);
		
		JButton upload = new JButton("Upload");
		upload.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				for (ImageData data:imageManager.getImageData()) {
					byte[] rawData = data.getData();
					String base64Data = DatatypeConverter.printBase64Binary(rawData);
					System.out.println(base64Data);
					
					JSONObject json = new JSONObject();
					JSONArray teamArray = new JSONArray();
					for (String team:data.getTeams())
						teamArray.add(team);
					json.accumulate("teams", teamArray);
					json.accumulate("data", base64Data);
					String postData = json.toString();
					System.out.println(postData);
					
					try {
						URL url = new URL("http://127.0.0.1:9000/photos");
						URLConnection conn = url.openConnection();
						conn.setDoOutput(true);//post
						conn.setRequestProperty("Content-Length", ""+postData.length());
						OutputStream out = conn.getOutputStream();
						PrintWriter pw = new PrintWriter(out);
						pw.print(postData);
						pw.close();
						Scanner s = new Scanner(conn.getInputStream());
						while (s.hasNextLine())
							System.out.println(s.nextLine());
						System.out.println();
						imageManager.removeImage(data.getFile());
					} catch (MalformedURLException e1) {
						e1.printStackTrace();
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}
			}
		});
		imagePanel.add(upload);
		
		JScrollPane imgScroll = new JScrollPane(imagePanel);
		this.add(imgScroll);
		
		imageManager = new ImageManagerImpl(imagePanelDropbox);
		imageSelector = new ImageSelector(imageManager);
		
		initMenu();
	}

	private void initMenu() {
		JMenuBar menuBar = new JMenuBar();
		
		//init File menu
		JMenu file = new JMenu("File");
		file.setMnemonic('F');
		JMenuItem selectImages = new JMenuItem("Select Images...");
		selectImages.setMnemonic('S');
		selectImages.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new Thread(new Runnable() {
					public void run() {
						imageSelector.selectImage();
					}
				}).start();
			}
		});
		file.add(selectImages);
		JMenuItem exit = new JMenuItem("Exit");
		exit.setMnemonic('X');
		exit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ScoutMasterDesktopClientFrame.this.dispose();
			}
		});
		file.add(exit);
		menuBar.add(file);
		
		this.setJMenuBar(menuBar);
	}
}
