package edu.mbhs.robot.scoutmaster9000.client.desktop.imageUpload;

import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.swing.JPanel;

/**
 * An implementation of ImageManager that keeps a list of the images and displays them (in ImagePanels) in a JPanel.
 * @author David Kaufman
 */
public class ImageManagerImpl implements ImageManager {
	private HashMap<File, ImageData> imageDataMap;
	private HashMap<File, ImagePanel> imagePanelMap;
	private JPanel imagePanelContainer;
	
	/**
	 * Creates a new Image Manager (adds/removes images from a Container)
	 * @param imagePanelContainer the container to put ImagePanels in
	 */
	public ImageManagerImpl(JPanel imagePanelContainer) {
		this.imagePanelContainer = imagePanelContainer;
		imageDataMap = new HashMap<File, ImageData>();
		imagePanelMap = new HashMap<File, ImagePanel>();
	}
	
	@Override
	public synchronized void addImage(File file) throws IOException {
		if (imageDataMap.containsKey(file))
			return;
		ArrayList<Byte> dataList = new ArrayList<Byte>();
		InputStream in = new BufferedInputStream(new FileInputStream(file));
		int val;
		while ((val=in.read())!=-1)
			dataList.add((byte)val);
		byte[] data = new byte[dataList.size()];
		for (int i=0;i<data.length;i++)
			data[i] = dataList.get(i);
		
		Iterator<ImageReader> readers = ImageIO.getImageReadersBySuffix(file.getName().substring(file.getName().lastIndexOf('.')+1));
		if (readers.hasNext()) {
			ImageReader reader = readers.next();
			reader.setInput(ImageIO.createImageInputStream(new ByteArrayInputStream(data)));
			BufferedImage img = reader.read(0);
			ImagePanel ip = new ImagePanel(file, img);
			imagePanelMap.put(file, ip);
			imagePanelContainer.add(ip);
			imagePanelContainer.revalidate();
			imagePanelContainer.repaint();
			
			ImageData imgData = new ImageData(file, data, ip);
			imageDataMap.put(file,imgData);
		}
	}

	@Override
	public synchronized void removeImage(File file) {
		if (imagePanelMap.containsKey(file)) {
			imagePanelContainer.remove(imagePanelMap.get(file));
			imagePanelContainer.revalidate();
			imagePanelContainer.repaint();
			
			imageDataMap.remove(file);
			imagePanelMap.remove(file);
		}
	}

	@Override
	public synchronized List<ImageData> getImageData() {
		return new LinkedList<ImageData>(imageDataMap.values());
	}

}
