package edu.mbhs.robot.scoutmaster9000.client.desktop.imageUpload;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

/**
 * A panel that displays an image, a team id field, and a caption text area.
 * @author David Kaufman
 */
public class ImagePanel extends JPanel {
	private static final long serialVersionUID = 1L;
	private static final int PADDING = 30;

	private File file;
	private BufferedImage image;
	private JTextField teamNumbers;
	private JTextArea caption;

	/**
	 * Constructs a new ImagePanel with the provided image and the file the image came from (for identification purposes).
	 * @param imageFile the image
	 * @param img the file
	 * @throws IllegalArgumentException if the image is null
	 */
	public ImagePanel(File imageFile, BufferedImage img) throws IllegalArgumentException {
		file = imageFile;
		image = img;
		if (image==null)
			throw new IllegalArgumentException("Invalid image");
		teamNumbers = new JTextField(8);
		caption = new JTextArea("", 3, 20);
		caption.setWrapStyleWord(true);
		
		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		
		JPanel textPanel = new JPanel();
		textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.Y_AXIS));
		
		textPanel.add(new JLabel("Teams"));
		JPanel teamsPanel = new JPanel();
		teamsPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		teamsPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		teamsPanel.add(teamNumbers);
		textPanel.add(teamsPanel);

		JLabel captionLabel = new JLabel("Caption");
		captionLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		textPanel.add(captionLabel);
		
		JPanel captionPanel = new JPanel();
		captionPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		captionPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		captionPanel.add(caption);
		textPanel.add(captionPanel);

		textPanel.add(Box.createVerticalStrut(PADDING));
		this.add(textPanel);
		
		JPanel imagePanel = new JPanel() {
			private static final long serialVersionUID = 1L;

			public void paint(Graphics g) {
				int panelWidth = getWidth(), panelHeight = getHeight() - PADDING;
				double imgAspect = image.getWidth()/image.getHeight();
				double panelAspect = panelWidth/panelHeight;
				int imgWidth, imgHeight;
				if (imgAspect>panelAspect) {
					//width is the constraining factor
					imgWidth = getWidth();
					imgHeight = (int)(imgWidth/imgAspect);
				} else {
					imgHeight = panelHeight;
					imgWidth = (int)(imgAspect * imgHeight);
				}
				
				g.drawImage(image,
						(panelWidth-imgWidth)/2,//dx1
						(panelHeight-imgHeight)/2,//dy1
						(panelWidth+imgWidth)/2,//dx2
						(panelHeight+imgHeight)/2,//dy2
						0, 0, image.getWidth(), image.getHeight(), null);
			}
		};
		
		this.add(imagePanel);
	}

	/**
	 * @return the caption
	 */
	public String getCaption() {
		return caption.getText();
	}

	/**
	 * @return the image
	 */
	public BufferedImage getImage() {
		return image;
	}

	/**
	 * @return the file
	 */
	public File getFile() {
		return file;
	}

	/**
	 * @return the teams (split on spaces and commas)
	 */
	public String[] getTeams() {
		//replace spaces with commas, remove duplicate commas, replace with spaces
		return teamNumbers.getText().replaceAll("\\s",",").replaceAll(",,+"," ").split(" ");
	}	
}
