package edu.mbhs.robot.scoutmaster9000.client.desktop.imageUpload;

import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

/**
 * Handles the selection of images (with a file chooser)
 * @author David Kaufman
 */
public class ImageSelector {
	private File file = null;//TODO init from saved settings
	private JFileChooser fileChooser;
	private ImageManager imageManager;

	/**
	 * Creates an ImageSelector that uses the provided ImageManager to process the selected images
	 * @param imageManager the ImageManager to use
	 */
	public ImageSelector(ImageManager imageManager) {
		this.imageManager = imageManager;
		
		//init file chooser
		if (file==null)
			fileChooser = new JFileChooser();
		else
			fileChooser = new JFileChooser(file);
		
		//images filter
		fileChooser.addChoosableFileFilter(new FileFilter() {
			public boolean accept(File file) {
				return isImage(file);
			}
			
			public String getDescription() {
				return "Images (PNG, JPG, GIF)";
			}
		});
		
		//all files filter
		fileChooser.addChoosableFileFilter(new FileFilter() {
			public boolean accept(File pathname) {
				return true;
			}
			
			public String getDescription() {
				return "All Files";
			}
		});
		
		fileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		fileChooser.setMultiSelectionEnabled(true);
	}

	/**
	 * Prompts the user to select one or more images. Handles the images with the ImageManager.
	 */
	public void selectImage() {
		if (JFileChooser.APPROVE_OPTION != fileChooser.showOpenDialog(null))
				return;
		
		File[] files = fileChooser.getSelectedFiles();
		for (File file:files) {
			if (file.isFile()) {
				if (isImage(file)) {
					try {
						imageManager.addImage(file);
					} catch (IOException e) {
						//invalid image or IO failure
						//TODO display some sort of message
					}
				}
			}
			else {//directory
				for (File f:file.listFiles()) {
					if (isImage(file)) {
						try {
							imageManager.addImage(f);
						} catch (IOException e) {
							//invalid image or IO failure
							//TODO display some sort of message
						}
					}
				}
			}
		}
	}

	/**
	 * Used to determine if a file is an image or not.
	 * @param file the file
	 * @return true if it is an image, false if it isn't
	 */
	private boolean isImage(File file) {
		return file.isDirectory() 
				|| file.getName().toLowerCase().endsWith(".png")
				|| file.getName().toLowerCase().endsWith(".jpg")
				|| file.getName().toLowerCase().endsWith(".gif");
	}
}
